def range(arr: list) -> float:
    """Returns the range of a list (the difference between the lowest value and the highest)

    Args:
        arr (list): array of numbers

    Returns:
        float: the range of the array (difference between lowest and highest)
    """
    try:
        return float(max(arr) - min(arr))
    except:
        raise TypeError("Please provide an array with numbers only!")



import random

def shuffle(array: list) -> list:
    """
    Shuffles a list

    Args:
        array (list): the list to be shuffled

    Returns:
        list: the shuffled list
    """
    new_array = []
    indexes_of_array = [x for x in range(0, len(array))]
    
    for item in array:
        random_index = random.randrange(0, len(indexes_of_array))
        new_array.append(array[indexes_of_array[random_index]])
        
        indexes_of_array.remove(indexes_of_array[random_index])
        
    return new_array



def remove_duplicate(array: list) -> list:
    """A function that removes duplicates from an array

    Args:
        array (list): Can be any length list, with any type

    Returns:
        array (list): Returns a list with unique characters (all duplicates removed)
    """
    new_array = []

    for item in array:
        if item not in new_array:
            new_array.append(item)

    return new_array



from utilitilib.strings.replace_one import replace_one

def format_num(num: int, custom_comma=",", custom_negative="-", custom_decimal=".") -> str:
    """Makes a number more readable by adding commas or by shortening it.

    Args:
        num (int): the number that needs to be formatted or shortened (more readable)

    Returns:
        str: the formatted or shortened string
    """
    formatted_num = ("{:,}".format(num))
    formatted_num = (
        formatted_num
        .replace(",", custom_comma)
        .replace("-" if formatted_num[0] == "-" else "", custom_negative if formatted_num[0] == "-" else "")
    )
    return replace_one(formatted_num, ".", custom_decimal)



import unittest

from utilitilib.strings import (
    are_anagrams,
    censor_string,
    format_num,
    replace_one,
    to_title_case,
    remove_whitespace,
)

class TestStrings(unittest.TestCase):
    def setUp(self): 
        self.anagram_test_cases = [
            [["silent", "listen"], True],
            [["Hello", "World"], False],
            [["cinema", "iceman"], True],
        ]

        self.censor_default_valid_test_cases = [
            ["oh fuck", "oh ****"],
            ["oh shit", "oh ****"],
            ["you stupid bitch", "you stupid *****"],
            ["I FUCKING HATE THIS STUPID ASS JOB OMG SHIT", "i ****ing hate this stupid *** job omg ****"],
            ["nice cock :wink:", "nice **** :wink:"],
            ["dicking", "****ing"]
        ]
        
        self.custom_char_test_cases = [
            ["oh fuck you", ".", "oh .... you"],
            ["whats ur sex", "$", "whats ur $$$"],
            ["you stupid fucking bitch, cunt", "", "you stupid ing , "]
        ]

        self.custom_char_amount_test_cases = [
            ["oh fuck you", 1, "oh * you"],
            ["whats ur sex", 10, "whats ur **********"],
            ["you stupid fucking bitch, cunt", 3, "you stupid ***ing ***, ***"]
        ]
        
        self.custom_swear_word_test_cases = [
            ["kill yourself idiot", ["kill", "idiot"], "**** yourself *****"],
            ["fuck you bitch", ["something", "random"], "fuck you bitch"],
            ["you stupid fucking bitch, cunt", [], "you stupid ****ing *****, ****"],
            ["idiot", ["idiot"], "*****"]
        ]
        
        self.custom_addition_swear_word_cases = [
            ["kill yourself you fucking idiot", ["kill", "idiot"], "**** yourself you ****ing *****"],
            ["fuck you, you stupid random bitch", ["something", "random"], "**** you, you stupid ****** *****"],
            ["you stupid fucking bitch, cunt", [], "you stupid ****ing *****, ****"],
        ]

        self.format_valid_test_cases_with_default = [
            [1000, "1,000"],
            [10000, "10,000"],
            [100000, "100,000"],
            [1000000, "1,000,000"],
            [10000000, "10,000,000"],
            [100000000, "100,000,000"],
            [1000000000, "1,000,000,000"],
            [10000000000, "10,000,000,000"],
            [100000000000, "100,000,000,000"],
            [-1000, "-1,000"],
            [-10000, "-10,000"],
            [-100000, "-100,000"],
            [-1000000, "-1,000,000"],
            [-10000000, "-10,000,000"],
            [-100000000, "-100,000,000"],
            [-1000000000, "-1,000,000,000"],
            [-10000000000, "-10,000,000,000"],
            [-100000000000, "-100,000,000,000"],
            [234, "234"],
            [10, "10"],
            [1, "1"],
            [-32542.32, "-32,542.32"]
        ]
        
        self.format_valid_test_cases_custom = [
            [1000, "1.000", ".", ".", "-"],
            [10000.67, "10,000,67", ",", ",", "-"],
            [-100000, "+100BALLS000", "BALLS", ".", "+"],
            [1000000, "13.140003.14000", "3.14", ".", "-"],
            [10000000, "10,000,000", ",", ".", "-"],
            [-53120.32, "NEGATIVE 53:120;32", ":", ";", "NEGATIVE "]
        ]

        self.replace_one_valid_cases = [
            ["hello world", "o", ".", "hello w.rld"],
            ["10,000.67", ".", ",", "10,000,67"]
        ]

        self.to_title_case_test_cases = [
            ["why i pooped my pants", "Why I Pooped My Pants"],
            ["I AM SO DUMB", "I Am So Dumb"],
            ["Hello My Name Is", "Hello My Name Is"],
            ["", ""]
        ]

        self.remove_whitespace_test_cases = [
            ["Hello World", "HelloWorld"],
            ["I am an idiot   fr        ", "Iamanidiotfr"],
            ["   ", ""]
        ]

    def test_anagram_valid_cases(self):
        for x in self.anagram_test_cases:
            self.assertEqual(are_anagrams.are_anagrams(x[0][0], x[0][1]), x[1])

    def test_censor_valid_cases(self):
        for x in self.censor_default_valid_test_cases:
            self.assertEqual(censor_string.censor_string(x[0]), x[1])

    def test_censor_custom_test_cases(self):
        for x in self.custom_char_test_cases:
            self.assertEqual(censor_string.censor_string(x[0], censor_char=x[1]), x[2])

        for x in self.custom_char_amount_test_cases:
            self.assertEqual(censor_string.censor_string(x[0], censor_char_amount=x[1]), x[2])
        
        for x in self.custom_swear_word_test_cases:
            self.assertEqual(censor_string.censor_string(x[0], custom_swear_words=x[1]), x[2])
        
        for x in self.custom_addition_swear_word_cases:
            self.assertEqual(censor_string.censor_string(x[0], addition_swear_words=x[1]), x[2])

    
    def test_format_test_valid_cases(self):
        for x in self.format_valid_test_cases_with_default:
            self.assertEqual(format_num.format_num(x[0]), x[1])
           
        for x in self.format_valid_test_cases_custom:
            self.assertEqual(format_num.format_num(x[0], custom_comma=x[2], custom_decimal=x[3], custom_negative=x[4]), x[1])

        #self.assertEqual(format_num.format_num(23000.32, custom_comma=".", custom_decimal="x"), "23.000x32")

    def test_replace_one_valid_cases_method(self):
        for x in self.replace_one_valid_cases:
            self.assertEqual(replace_one.replace_one(x[0], x[1], x[2]), x[3])

    def test_to_title_case_valid_cases(self):
        for x in self.to_title_case_test_cases:
            self.assertEqual(to_title_case.to_title_case(x[0]), x[1])

    def test_remove_whitespace_valid_cases(self):
        for x in self.remove_whitespace_test_cases:
            self.assertEqual(remove_whitespace.remove_whitespace(x[0]), x[1])

if __name__ == "__main__":
    unittest.main()




from utilitilib.math.mean import mean

def median(arr: list) -> float:
    """Finds the median (middle value) from a list of numbers

    Args:
        arr (list): the list that contains the numbers

    Returns:
        float: the median (or middle) value from that list
    """
    sorted_list = sorted(arr)
    
    if len(sorted_list) == 0:
        return None
    elif len(sorted_list) == 1:
        return sorted_list[0]
    elif len(sorted_list) % 2 != 0:
        return sorted_list[len(sorted_list) // 2]
    else:
        return mean([sorted_list[len(sorted_list)//2 - 1], sorted_list[len(sorted_list)//2]])



import unittest

from utilitilib.arrays import (
    chunk_list,
    remove_all,
    remove_duplicate,
    shuffle,
    unique_value_counter
)

class TestArrays(unittest.TestCase):
    def setUp(self):
        self.chunk_list_tests = [
            [[1, 2, 3, 4, 5, 6, 7, 8, 9], 3, [[1, 2, 3], [4, 5, 6], [7, 8, 9]]],
            [[1, 2, 3, 4, 5, 6, 7, 8, 9], 4, [[1, 2, 3, 4], [5, 6, 7, 8], [9]]],
            [[1, 2, 3, 4, 5, 6, 7], 2, [[1, 2], [3, 4], [5, 6], [7]]],
            [[],1, []],
            [[1, 2, 3, 4, 5], 1, [[1], [2], [3], [4], [5]]]
        ]

        self.remove_all_test_cases = [
            [[[3,7,2,2,1,1,11,1], 1], [3,7,2,2,11]],
            [[[5,5,5,5,5,5,5,5], 5], []],
            [[[1235,345345,1235457,6234], 6234], [1235,345345,1235457]],
            [[["something", "this", "is", "something", "cool"], "something"], ["this", "is", "cool"]],
            [[["a", "b", "c", "d"], "a"], ["b", "c", "d"]],
            [[[True, True, False], True], [False]],
            [[[], 5], []],
        ]

        self.remove_expected_test_cases = [
            [[1,2,1,1,1,2,2,2,3,3,3], [1,2,3]],
            [[4,4,4,8,8,32,32,64,1,65, 100], [4,8,32,64,1,65,100]],
            [[1,2,3,4,5], [1,2,3,4,5]],
            [[1,1,1,1,1,1,1,1,1,1,1], [1]],
            [["true", "true", "balls", "nuts", "nuts", "nuts"], ["true", "balls", "nuts"]],
            [[True, True, True, False, False], [True, False]],
            [[1,1,1,2,2,2,3], [1,2,3]],
            [["Hello", "Hello", True, False, False, False, False, 2,2,2, 3], ["Hello", True, False, 2, 3]],
            [[], []]
        ]
        
        self.remove_not_expected_test_cases = [
            [[1,1,1,2,2,2,3,3,3], [1,1,1,2,2,2,3,3,3]],
        ]

        self.shuffle_expected_test_cases = [
            [[1,2,3,4,5], [1,2,3,4,5]],
            [[3,5,6,2,1], [6,2,1,6,2]],
            [[0,231,323123,534534,312312], [0,231,323123,534534,312312]],
            [[True, False, 1,2, "hello", "world"], [2,1,True, False,"world", "hello"]],
        ]
        
        self.shuffle_not_expected_test_cases = [
            [[], []]
        ]

        self.unique_value_valid_test_cases = [
            [[1,2,3,1,2,3,4,5,1,2,2], {2:4,1:3,3:2,4:1,5:1}],
            [[0,0,0,0,0,0], {0:6}],
            [[], {}],
            [['a','b','a','c','b','a','d'], {'a':3,'b':2,'c':1,'d':1}],
            [[1,2,3,3,3,3,'a','a'], {1:1,2:1,3:4,"a":2}],
            [["True", "False", "True", "None", "None", "None", 5, 5, 69], {"True": 2, "False": 1, "None": 3, 5: 2, 69: 1}]
        ]

    def test_chunk_list(self):
        for x in self.chunk_list_tests:
            self.assertEqual(chunk_list.chunk_list(x[0], x[1]), x[2])

    def test_remove_all(self):
        for x in self.remove_all_test_cases:
            self.assertEqual(remove_all.remove_all(x[0][0], x[0][1]), x[1])

    def test_remove_duplicate(self):
        for test in self.remove_expected_test_cases:
            self.assertEqual(remove_duplicate.remove_duplicate(test[0]), test[1])

        for test in self.remove_not_expected_test_cases:
            self.assertNotEqual(remove_duplicate.remove_duplicate(test[0]), test[1])

    def test_shuffle(self):
        for test in self.shuffle_expected_test_cases:
            self.assertNotEqual(shuffle.shuffle(test[0]), test[1])
            self.assertEqual(len(shuffle.shuffle(test[0])), len(test[1]))
            self.assertTrue(shuffle.shuffle(test[0])[2] in test[0])

        for test in self.shuffle_not_expected_test_cases:
            self.assertEqual(shuffle.shuffle(test[0]), test[1])
            self.assertEqual(len(shuffle.shuffle(test[0])), len(test[1]))

    def test_unique_value(self):
        for x in self.unique_value_valid_test_cases:
            self.assertEqual(unique_value_counter.unique_value_counter(x[0]), x[1])

if __name__ == "__main__":
    unittest.main()



import unittest

from utilitilib.math import (
    mean,
    mode,
    range,
    median
)

class TestMath(unittest.TestCase):
    def setUp(self): 
        self.test_mean_cases = [
            [[1,1,1,1,1,1], 1],
            [[15, 13, 18, 16, 14, 17, 12], 15],
            [[6, 2, 3], 11/3],
            [[6.5, 2.3, 75.3, 24.6, 2.22, 634.2231, 582.02, 69.0], 174.5203875],
            [[-5, -2, 2, 4, -2], -0.6],
            [[True, False, True, False, False, True, False, False], 0.375],
        ]
        
        self.test_invalid_mean_cases = [
            [],
            ["dasdas", 42, 321,21],
            1,
            [[],[],[],[]],
        ]

        self.median_valid_cases = [
            [[1, 3, 5, 7, 9], 5],
            [[2,4,6,6], 5.0],
            [[6,3,7,5,1,3,5], 5],
            [[-5, -3, 0, 2, 4], 0],
            [[1, 1, 2, 2, 3, 3], 2],
            [[5], 5],
            [[], None]
        ]

        self.mode_valid_cases = [
            [[2,2,3,1], 2],
            [[1,1,2,3,3,3], 3],
            [[5,2,6,6,2,1,1,3,8,8,8,8], 8],
            [[6,231,6324,1272,53,31,643], None],
            [[2,1,3,4,5,6,7], None],
            [[5234234,28736482734,5312, 5312, 5234234], [5234234, 5312]],
            [[5,5,3,3,2,1,65,8,6], [5,3]],
            [["idot", "hello", "bozo", "bozo", "blue", "ping", "ping", "bozo", "bozo"], "bozo"],
            [[True, True, False, True], True],
            [["idot", "idot", 2, 3,3], [3,"idot"]],
        ]
        
        self.mode_invalid_cases = [
            [283428934283649862836482638462873648268346826348623984628364892634, 2],
            123,
        ]

        self.valid_range_test_cases = [
            [[1,2,3,4,5,6],5],
            [[431,235,21,52,342,532,21],511],
            [[8,11,5,9,7,6,3616], 3611],
            [[3,4,5,6,7,8,9], 6],
            [[13, 13, 14, 14, 14, 15, 15, 15, 15, 16, 16, 16], 3],
        ]
        
        self.invalid_range_test_cases = [
            [[1,2,3,4, "float"], 3]
        ]

    def test_valid_mean(self):
        for test in self.test_mean_cases:
            self.assertEqual(mean.mean(test[0]), test[1])
    
    def test_invalid_mean(self):
        for test in self.test_invalid_mean_cases:
            with self.assertRaises(Exception):
                mean.mean(test)

    def test_valid_median(self):
        for test_case in self.median_valid_cases:
            self.assertEqual(median.median(test_case[0]), test_case[1])

    def test_valid_mode(self):
        for test in self.mode_valid_cases:
            self.assertEqual(mode.mode(test[0]), test[1])
    
    def test_invalid_mode(self):
        for test in self.mode_invalid_cases:
            with self.assertRaises(Exception):
                mode.mode(test)

    def test_valid_range(self):
        for x in self.valid_range_test_cases:
            self.assertEqual(range.range(x[0]), x[1])
            
    def test_invalid_range(self):
        for x in self.invalid_range_test_cases:
            with self.assertRaises(TypeError):
                range.range(x[0])

if __name__ == "__main__":
    unittest.main()




def censor_string(string: str, censor_char: str="*", custom_swear_words: list= [], addition_swear_words: list=[], censor_char_amount: int=0) -> str:
    """Censors all the swear words in a string

    Args:
        string (str): the string that needs to be censored
        censor_char (str): the character that replaces the swear word in the string
        custom_swear_words (list): a list of your own words that you would like to censor (replaces the default list of swear words)
        addition_swear_words (list): a list of words that get added to the already existing default swear words

    Returns:
        str: the fully censored string
    """
    swear_words = ([
        "fuck",
        "shit",
        "bitch",
        "ass",
        "cunt",
        "dick",
        "arse",
        "bastard",
        "tits",
        "penis",
        "cock",
        "bullshit",
        "pussy",
        "motherfucker",
        "sex",
        "fucking"
        "shitty",
        "nigger",
        "nigga",
        "niggers",
        "niggas"
    ] if len(custom_swear_words) == 0 else custom_swear_words) + addition_swear_words
    
    new_string = string.lower()
    
    for x in swear_words:
        new_string = new_string.replace(x, "".join([censor_char for y in x] if censor_char_amount <= 0 else [censor_char for y in range(0, censor_char_amount)]))

    return new_string



import unittest
from datetime import date

from utilitilib.dateandtime import (
    days_between_dates,
)

class TestDateAndTime(unittest.TestCase):
    def setUp(self):
        self.days_between_dates_cases = [
            [[date(2020, 5, 6), date(2024, 6, 5)], 1491],         
            [[date(2024, 1, 1), date(2024, 1, 2)], 1]
        ]

    def test_days_between_dates(self):
        for x in self.days_between_dates_cases:
            self.assertEqual(days_between_dates.days_between_dates(x[0][0], x[0][1]), x[1])

if __name__ == "__main__":
    unittest.main()



from typing import List, Union

def mean(array: List[Union[int, float]]) -> Union[int, float]:
    """Calculates the mean of all the numbers in a list

    Args:
        array (list): List containing numbers

    Returns:
        number: returns a int/float  
    """
    
    try:
        return (sum(array))/len(array)
    except Exception as e:
        raise e
    



def remove_all(arr: list, item: any) -> list:
    """Removes all specified elements from a list

    Args:
        arr (list): the array that needs the object to be removed from
        item (any): the item/object that needs to get removed from the array.

    Returns:
        list: the new list (all elements removed from it)
    """
    new_list = []
    
    
    for i in arr:
        if i != item:
            new_list.append(i)
            
    return new_list



import datetime

def days_between_dates(date1: datetime.date, date2: datetime.date) -> int:
    
    """Returns the amount of days between two dates
    Args:
        date1 (): 
        date2 (): 

    Returns:
        int: the amount of days between the two dates.
    """
    return (date2 - date1).days




from utilitilib.arrays.remove_duplicate import remove_duplicate

def mode(arr: list) -> any:
    """Returns the mode from a given list

    Args:
        array (list): A list containing any data type

    Returns:
        any: The mode of that current list
    """

    occurences = {}

    for item in arr:
        if item in occurences:
            occurences[item] += 1
        else:
            occurences[item] = 1            
    
    value = list(occurences.keys())[list(occurences.values()).index(find_all_max(list(occurences.values()))[0])]
    occ = max(occurences.values())
    
    return None if occ == 1 else value

def find_all_max(arr: list) -> list:
    max_value = max(arr)
    val = [max_value]
    b = False
    
    while b == False:
        try:  
            arr.remove(max_value)
            if max(arr) == max_value:
                val.append(max(arr))
            else:
                b = True
        except:
            b = True
    
    return val



def remove_whitespace(text: str) -> str:
    """Removes any whitespace in a string and returns the string without any whitespace

    Args:
        text (str): the input string
    
    Returns:
       str: the new string that does not contain any whitespace
    """
    return text.replace(" ", "")




from setuptools import setup, find_packages

with open("./README.md", "r", encoding="utf-8") as f:
    long_desc_md = f.read()   

setup (
    name="utilitilib",
    version="0.0.1",
    author="Diman Gal Henege",
    author_email="dimanchansilu@gmail.com",
    description="A python utility package that provides commonly used functionalities or helper methods.",
    long_description=long_desc_md,
    long_description_content_type="text/markdown",
    url="https://github.com/oDqnger/PyUtilities",
    packages=find_packages(),
    requires=[],
    keywords=["python", "helper", "functions", "utility", "utilities", "algorithms", "data types"],
    license="MIT License",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.0",
    ]
)



def are_anagrams(str1: str, str2: str) -> bool:
    """Checks if two strings are anagrams of each other, i.e., if they contain the same characters with the same frequency, but not necessarily in the same order.

    Args:
        str1 (str): the first string
        str2 (str): the second string you need to compare the first string to

    Returns:
        bool: True if the two strings are anagrams, False if not.
    """
    if len(str1) == len(str2):
        for x in str1:
            if str2.count(x) > 1 or str2.count(x) == 0:
                return False
            
        return True
                
    return False



from .remove_duplicate import remove_duplicate

def unique_value_counter(arr: list) -> dict:
    """Generates a dictionary that contains occurences of unique values

    Args:
        arr (list): the list that the dict needs to be generated on

    Returns:
        dict: a dictionary that contains the occurences of each unique value in a list
    """
    unique_arr = remove_duplicate(arr)
    value_counter_dict = {}
    temp = 0
    
    for unique in unique_arr:
        for item in arr:
            if unique == item:
                temp += 1
                value_counter_dict[unique] = temp
        
        temp = 0
        
    return value_counter_dict



def replace_one(string: str, replace_char: str, char) -> str:
    """Replaces one occurence of a value, instead of multiple occurences

    Args:
        string (str): the string that has the characters that needs to be replaced
        replace_char (str): the character that needs to be replaced in the string
        char (str): the character that replace_char needs to be replaced with

    Returns:
        str: the newly created string with the char removed from it
    """
    string_list = list(string)
    if (string.rfind(replace_char) != -1):
        string_list[string.rfind(replace_char)] = char
        return "".join(string_list)
    else:
        return string



def chunk_list(lst: list, size: int) -> list:
    """Chunks a list into smaller lists of a specified size

    Args:
        lst (list): original list to be chunked
        size (int): size of each chunk
        
    Returns:
        list: a list containing all the chunks
    """
    new_list = []
    temp_list = []
    temp_num = 0
    
    for x in lst:
        temp_list.append(x)
        temp_num += 1
        
        if (temp_num == size):
            temp_num = 0
            new_list.append(temp_list)
            temp_list = []
            
    if (len(temp_list) != 0):
        new_list.append(temp_list)
    
    return new_list



def to_title_case(sentence: str) -> str:
    """Converts the first letter of each word in a sentence to uppercase, while converting all other letters to lowercase.

    Args:
        sentence (str): the string/sentence you want to change

    Returns:
        str: the string/sentence with the first letter of each word uppercased and all letters lowercased
    """
    try: 
        sentence = map(lambda item: item[0].upper()+item[1::].lower(),sentence.split(" "))
        return " ".join(sentence)
    except IndexError:
        return ""
